Test elliptic curves more thoroughly.
[u/mdw/catacomb] / ec.h
diff --git a/ec.h b/ec.h
index 105838c..d398f4a 100644 (file)
--- a/ec.h
+++ b/ec.h
@@ -1,6 +1,6 @@
 /* -*-c-*-
  *
- * $Id: ec.h,v 1.4 2003/05/15 23:25:59 mdw Exp $
+ * $Id: ec.h,v 1.7 2004/03/23 15:19:32 mdw Exp $
  *
  * Elliptic curve definitions
  *
 /*----- Revision history --------------------------------------------------* 
  *
  * $Log: ec.h,v $
+ * Revision 1.7  2004/03/23 15:19:32  mdw
+ * Test elliptic curves more thoroughly.
+ *
+ * Revision 1.6  2004/03/22 02:19:10  mdw
+ * Rationalise the sliding-window threshold.  Drop guarantee that right
+ * arguments to EC @add@ are canonical, and fix up projective implementations
+ * to cope.
+ *
+ * Revision 1.5  2004/03/21 22:52:06  mdw
+ * Merge and close elliptic curve branch.
+ *
+ * Revision 1.4.4.3  2004/03/21 22:39:46  mdw
+ * Elliptic curves on binary fields work.
+ *
+ * Revision 1.4.4.2  2004/03/20 00:13:31  mdw
+ * Projective coordinates for prime curves
+ *
+ * Revision 1.4.4.1  2003/06/10 13:43:53  mdw
+ * Simple (non-projective) curves over prime fields now seem to work.
+ *
  * Revision 1.4  2003/05/15 23:25:59  mdw
  * Make elliptic curve stuff build.
  *
@@ -80,27 +100,40 @@ typedef struct ec_mulfactor {
   mp *exp;                             /* The exponent */
 } ec_mulfactor;
 
-/* --- Elliptic curve operations --- */
+/* --- Elliptic curve operations --- *
+ *
+ * All operations (apart from @destroy@ and @in@) are guaranteed to be
+ * performed on internal representations of points.
+ *
+ * (Historical note.  We used to guarantee that the second to @add@ and @mul@
+ * was the output of @in@ or @fix@, but this canonification turned out to
+ * make the precomputation in @ec_exp@ too slow.  Projective implementations
+ * must therefore cope with a pair of arbitrary points.)
+ */
 
 typedef struct ec_ops {
   void (*destroy)(ec_curve */*c*/);
   ec *(*in)(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
   ec *(*out)(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
+  ec *(*fix)(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
   ec *(*find)(ec_curve */*c*/, ec */*d*/, mp */*x*/);
   ec *(*neg)(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
   ec *(*add)(ec_curve */*c*/, ec */*d*/, const ec */*p*/, const ec */*q*/);
   ec *(*sub)(ec_curve */*c*/, ec */*d*/, const ec */*p*/, const ec */*q*/);
   ec *(*dbl)(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
+  int (*check)(ec_curve */*c*/, const ec */*p*/);
 } ec_ops;
 
 #define EC_IN(c, d, p)         (c)->ops->in((c), (d), (p))
-#define EC_OUT(c, d, p)                (c)->ops->in((c), (d), (p))
+#define EC_OUT(c, d, p)                (c)->ops->out((c), (d), (p))
+#define EC_FIX(c, d, p)                (c)->ops->fix((c), (d), (p))
 
 #define EC_FIND(c, d, x)       (c)->ops->find((c), (d), (x))
 #define EC_NEG(c, d, x)                (c)->ops->neg((c), (d), (x))
 #define EC_ADD(c, d, p, q)     (c)->ops->add((c), (d), (p), (q))
 #define EC_SUB(c, d, p, q)     (c)->ops->sub((c), (d), (p), (q))
 #define EC_DBL(c, d, p)                (c)->ops->dbl((c), (d), (p))
+#define EC_CHECK(c, p)         (c)->ops->check((c), (p))
 
 /*----- Simple memory management things -----------------------------------*/
 
@@ -205,33 +238,23 @@ extern ec *ec_setinf(ec */*p*/);
 
 extern ec *ec_copy(ec */*d*/, const ec */*p*/);
 
-/*----- Interesting arithmetic --------------------------------------------*/
-
-/* --- @ec_in@ --- *
+/* --- @ec_eq@ --- *
  *
- * Arguments:  @ec_curve *c@ = pointer to an elliptic curve
- *             @ec *d@ = pointer to the destination point
- *             @const ec *p@ = pointer to the source point
+ * Arguments:  @const ec *p, *q@ = two points
  *
- * Returns:    The destination point.
- *
- * Use:                Converts a point to internal representation.
+ * Returns:    Nonzero if the points are equal.  Compares external-format
+ *             points.
  */
 
-extern ec *ec_in(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
+#define EC_EQ(p, q)                                                    \
+    ((EC_ATINF(p) && EC_ATINF(q)) ||                                   \
+     (!EC_ATINF(p) && !EC_ATINF(q) &&                                  \
+      MP_EQ((p)->x, (q)->x) &&                                         \
+      MP_EQ((p)->y, (q)->y)))
 
-/* --- @ec_out@ --- *
- *
- * Arguments:  @ec_curve *c@ = pointer to an elliptic curve
- *             @ec *d@ = pointer to the destination point
- *             @const ec *p@ = pointer to the source point
- *
- * Returns:    The destination point.
- *
- * Use:                Converts a point to external representation.
- */
+extern int ec_eq(const ec *p, const ec *q);
 
-extern ec *ec_out(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
+/*----- Interesting arithmetic --------------------------------------------*/
 
 /* --- @ec_find@ --- *
  *
@@ -249,6 +272,19 @@ extern ec *ec_out(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
 
 extern ec *ec_find(ec_curve */*c*/, ec */*d*/, mp */*x*/);
 
+/* --- @ec_rand@ --- *
+ *
+ * Arguments:  @ec_curve *c@ = pointer to an elliptic curve
+ *             @ec *d@ = pointer to the destination point
+ *             @grand *r@ = random number source
+ *
+ * Returns:    The destination @d@.
+ *
+ * Use:                Finds a random point on the given curve.
+ */
+
+extern ec *ec_rand(ec_curve */*c*/, ec */*d*/, grand */*r*/);
+
 /* --- @ec_neg@ --- *
  *
  * Arguments:  @ec_curve *c@ = pointer to an elliptic curve
@@ -303,6 +339,18 @@ extern ec *ec_sub(ec_curve */*c*/, ec */*d*/,
 
 extern ec *ec_dbl(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
 
+/* --- @ec_check@ --- *
+ *
+ * Arguments:  @ec_curve *c@ = pointer to an elliptic curve
+ *             @const ec *p@ = pointer to the point
+ *
+ * Returns:    Zero if OK, nonzero if this is an invalid point.
+ *
+ * Use:                Checks that a point is actually on an elliptic curve.
+ */
+
+extern int ec_check(ec_curve */*c*/, const ec */*p*/);
+
 /* --- @ec_mul@, @ec_imul@ --- *
  *
  * Arguments:  @ec_curve *c@ = pointer to an elliptic curve
@@ -340,7 +388,7 @@ extern ec *ec_immul(ec_curve */*c*/, ec */*d*/,
 
 /*----- Standard curve operations -----------------------------------------*/
 
-/* --- @ec_idin@, @ec_idout@ --- *
+/* --- @ec_idin@, @ec_idout@, @ec_idfix@ --- *
  *
  * Arguments:  @ec_curve *c@ = pointer to an elliptic curve
  *             @ec *d@ = pointer to the destination
@@ -355,8 +403,9 @@ extern ec *ec_immul(ec_curve */*c*/, ec */*d*/,
 
 extern ec *ec_idin(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
 extern ec *ec_idout(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
+extern ec *ec_idfix(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
 
-/* --- @ec_projin@, @ec_projout@ --- *
+/* --- @ec_projin@, @ec_projout@, @ec_projfix@ --- *
  *
  * Arguments:  @ec_curve *c@ = pointer to an elliptic curve
  *             @ec *d@ = pointer to the destination
@@ -370,6 +419,7 @@ extern ec *ec_idout(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
 
 extern ec *ec_projin(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
 extern ec *ec_projout(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
+extern ec *ec_projfix(ec_curve */*c*/, ec */*d*/, const ec */*p*/);
 
 /* --- @ec_stdsub@ --- *
  *
@@ -402,7 +452,7 @@ extern void ec_destroycurve(ec_curve */*c*/);
 
 /* --- @ec_prime@, @ec_primeproj@ --- *
  *
- * Arguments:  @field *f@ = the underyling field for this elliptic curve
+ * Arguments:  @field *f@ = the underlying field for this elliptic curve
  *             @mp *a, *b@ = the coefficients for this curve
  *
  * Returns:    A pointer to the curve.
@@ -415,18 +465,20 @@ extern void ec_destroycurve(ec_curve */*c*/);
 extern ec_curve *ec_prime(field */*f*/, mp */*a*/, mp */*b*/);
 extern ec_curve *ec_primeproj(field */*f*/, mp */*a*/, mp */*b*/);
 
-/* --- @ec_bin@ --- *
+/* --- @ec_bin@, @ec_binproj@ --- *
  *
  * Arguments:  @field *f@ = the underlying field for this elliptic curve
  *             @mp *a, *b@ = the coefficients for this curve
  *
  * Returns:    A pointer to the curve.
  *
* Use:                Creates a curve structure for a non-supersingular elliptic
*             curve defined over a binary field.
  */
 
 extern ec_curve *ec_bin(field */*f*/, mp */*a*/, mp */*b*/);
+extern ec_curve *ec_binproj(field */*f*/, mp */*a*/, mp */*b*/);
 
 /*----- That's all, folks -------------------------------------------------*/
@@ -415,18 +465,20 @@ extern void ec_destroycurve(ec_curve */*c*/);
  * Returns:    A pointer to the curve.
  *
- * Use:                Creates a curve structure for a non-supersingular elliptic
- *             curve defined over a binary field.
+ * Use:                Creates a curve structure for an elliptic curve defined over
+ *             a binary field.  The @binproj@ variant uses projective
+ *             coordinates, which can be a win.
  */
 